<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" 
				xmlns:MSHelp="http://msdn.microsoft.com/mshelp"
        xmlns:mshelp="http://msdn.microsoft.com/mshelp"
				xmlns:ddue="http://ddue.schemas.microsoft.com/authoring/2003/5"
				xmlns:xlink="http://www.w3.org/1999/xlink"
        xmlns:msxsl="urn:schemas-microsoft-com:xslt"
         >

  <xsl:import href="globalTemplates.xsl"/>
  <xsl:template name="upperBodyStuff">
    <input type="hidden" id="userDataCache" class="userDataStyle"/>
    <input type="hidden" id="hiddenScrollOffset"/>
    
    <xsl:call-template name="commonImages"/>

    <xsl:call-template name="bodyHeader"/>
    
  </xsl:template>

  <xsl:template name="bodyHeader">
    <div id="header">
      <xsl:call-template name="bodyHeaderTopTable"/>

      <xsl:call-template name="bodyHeaderBottomTable"/>

      <xsl:if test="$languages != 'false'">
      <xsl:if test="($group='type' or $group='member' or $languageFilterSection) and not($pseudo) and (count($languages/language) &gt; 0)">
        <xsl:call-template name="languageSpan"/>
        </xsl:if>
      </xsl:if>

      <!-- include the member options dropdown on memberlist topics -->
      <xsl:if test="$group='list' and not($pseudo or $subgroup='DerivedTypeList')">
        <xsl:call-template name="membersOptionsSpan"/>
      </xsl:if>
    </div>
  </xsl:template>

  <xsl:template name="bodyHeaderTopTable">
    <table id="topTable">
      <tr id="headerTableRow1">
        <td align="left">
          <span id="runningHeaderText">
            <xsl:call-template name="runningHeader" />
          </span>
        </td>
      </tr>
      <tr id="headerTableRow2">
        <td align="left">
          <span id="nsrTitle">
            <xsl:call-template name="topicTitleDecorated"/>
          </span>
        </td>
      </tr>
      <tr id="headerTableRow3">
        <td>
          <xsl:call-template name="headerRowLinks"/>
        </td>
      </tr>
    </table>
  </xsl:template>

  <xsl:template name="headerRowLinks">
    <!-- most mref topics get autogenerated see also links to see also section -->
    
    <xsl:if test="$hasSeeAlsoSection">
      
      <a href="#seeAlsoToggle" onclick="OpenSection(seeAlsoToggle)">
        <xsl:text/>
        <include item="SeeAlso"/>
        <xsl:text/>
      </a>&#160;
      
    </xsl:if>
    
    <!-- include Example link if there's an Example section -->
    <xsl:choose>
      <xsl:when test="$examplesSection">
        <a href="#exampleToggle" onclick="OpenSection(exampleToggle)">
          <xsl:text/>
          <include item="Example"/>
          <xsl:text/>
        </a>&#160;
      </xsl:when>
    </xsl:choose>

    <!-- class, structure, and interface About topics get link to Members topic (unless the doc model has the all members lists on the type topic) -->
    <xsl:if test="/document/reference/topicdata/@allMembersTopicId">
      <referenceLink target="{/document/reference/topicdata/@allMembersTopicId}">
        <include item="allMembersTitle"/>
      </referenceLink>
    </xsl:if>

    <!--all members only -->
    <xsl:if test="$subgroup='members'">
        <xsl:variable name="visibility">
          <xsl:for-each select="/document/reference/elements/element">
          <xsl:choose>
            <xsl:when test=".//memberdata[@visibility = 'public']">
              <xsl:choose>
                <xsl:when test="not(apidata[@subsubgroup])">
                  <xsl:value-of select="concat('Public', apidata/@subgroup, ';')"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat('Public', apidata/@subsubgroup, ';')"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:when test=".//memberdata[@visibility = 'private'] and not(.//proceduredata[@virtual = 'true'])">
              <xsl:choose>
                <xsl:when test="not(apidata[@subsubgroup])">
                  <xsl:value-of select="concat('Private', apidata/@subgroup, ';')"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat('Private', apidata/@subsubgroup, ';')"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:when test=".//memberdata[@visibility = 'private'] and .//proceduredata[@virtual = 'true']">
              explicit;
            </xsl:when>
            <xsl:otherwise>
              <xsl:choose>
                <xsl:when test="not(apidata[@subsubgroup])">
                  <xsl:value-of select="concat('Protected', apidata/@subgroup, ';')"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat('Protected', apidata/@subsubgroup, ';')"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
          </xsl:for-each>
        </xsl:variable>
      
      
      <xsl:for-each select="/document/reference/elements/element">
        <xsl:variable name="subgroup" select="apidata/@subgroup"/>
        <xsl:variable name="subsubgroup" select="apidata/@subsubgroup" />
        <xsl:if test="not(preceding-sibling::element[apidata[@subgroup=$subgroup]]) and not($subsubgroup='attachedProperty' or $subsubgroup='attachedEvent') and string($subgroup)">
          <xsl:if test="not($subsubgroup)">
            <xsl:variable name="elementVisibility">
              <xsl:choose>
                <xsl:when test="contains($visibility, concat('Public', $subgroup))">
                  <xsl:value-of select="concat('public', $subgroup)"/>
                </xsl:when>
                <xsl:when test="contains($visibility, concat('Private', $subgroup))">
                  <xsl:value-of select="concat('private', $subgroup)"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat('protected', $subgroup)"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            
            <!-- add a link to the member list section for this subgroup -->
            <a href="#{$elementVisibility}Toggle" onclick="OpenSection({$elementVisibility}Toggle)">
              <xsl:text/>
              <include item="{$subgroup}Group"/>
              <xsl:text/>
            </a>&#160;
          </xsl:if>
        </xsl:if>
        <xsl:if test="not(preceding-sibling::element[apidata[@subsubgroup=$subsubgroup]]) and ($subsubgroup='attachedProperty' or $subsubgroup='attachedEvent') and string($subsubgroup)">
          <xsl:variable name="elementVisibility">
            <xsl:choose>
              <xsl:when test="contains($visibility, concat('Public', $subsubgroup))">
                <xsl:value-of select="concat('public', $subsubgroup)"/>
              </xsl:when>
              <xsl:when test="contains($visibility, concat('Private', $subsubgroup))">
                <xsl:value-of select="concat('private', $subsubgroup)"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="concat('protected', $subsubgroup)"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <!-- add a link to the member list section for this subgroup -->
          <a href="#{$elementVisibility}Toggle" onclick="OpenSection({$elementVisibility}Toggle)">
            <xsl:text/>
            <include item="{$subsubgroup}Group"/>
            <xsl:text/>
          </a>&#160;
        </xsl:if>
      </xsl:for-each>

      <xsl:if test="contains($visibility, 'explicit')">
        <a href="#ExplicitInterfaceImplementationToggle" onclick="OpenSection(ExplicitInterfaceImplementationToggle)">
          <xsl:text/>
          <include item="ExplicitInterfaceImplementation"/>
          <xsl:text/>
        </a>&#160;
      </xsl:if>
     </xsl:if>
        
  </xsl:template>


  <xsl:template name="bodyHeaderBottomTable">
    <table id="bottomTable" cellspacing="0" cellpadding="0">
      <tr>
        <td>
          <span onclick="ExpandCollapseAll(toggleAllImage)" style="cursor:default;" onkeypress="ExpandCollapseAll_CheckKey(toggleAllImage, event)" tabindex="0">
            <img ID="toggleAllImage" class="toggleAll">
              <includeAttribute name="src" item="iconPath">
                <parameter>collall.gif</parameter>
              </includeAttribute>
              <includeAttribute name="alt" item="collapseAllImage" />
            </img>
            <label id="collapseAllLabel" for="toggleAllImage" style="display: none;">
              <include item="collapseAll"/>
            </label>
            <label id="expandAllLabel" for="toggleAllImage" style="display: none;">
              <include item="expandAll"/>
            </label>
            <xsl:text>&#160;</xsl:text>
          </span>

          <xsl:if test="$languages != 'false'">
          <xsl:if test="($group='type' or $group='member' or $languageFilterSection) and not($pseudo) and (count($languages/language) &gt; 0)">
            <xsl:call-template name="languageFilterToolTip"/>
            </xsl:if>
          </xsl:if>

          <!-- include the member options dropdown on memberlist topics -->
          <xsl:if test="$group='list' and not($pseudo or $subgroup='DerivedTypeList')">
            <xsl:call-template name="membersOptionsFilterToolTip"/>
          </xsl:if>
        </td>
      </tr>
    </table>
  </xsl:template>

  <xsl:template name="membersOptionsSpan">
    <div id="membersOptionsSpan">
      <input id="inheritedCheckbox" type='checkbox' name="membersOptionsFilter" onClick="SetMembersOptions(this)"/>
      <label class="membersOptionsFilter" for="inheritedCheckbox">
        <include item="includeInheritedMembers"/>
      </label>
      <br/>
      <input id="protectedCheckbox" type='checkbox' name="membersOptionsFilter" onClick="SetMembersOptions(this)"/>
      <label class="membersOptionsFilter" for="protectedCheckbox">
        <include item="includeProtectedMembers"/>
      </label>
      <br/>
      <input id="netcfCheckbox" type='checkbox' name="membersOptionsFilter" onClick="SetMembersOptions(this)"/>
      <label class="membersOptionsFilter" for="netcfCheckbox">
        <include item="netcfMembersOnly"/>
      </label>
      <br/>
      <!-- add a checkbox for XNA filtering -->
      <input id="netXnaCheckbox" type='checkbox' name="membersOptionsFilter" onClick="SetMembersOptions(this)"/>
      <label class="membersOptionsFilter" for="netXnaCheckbox">
        <include item="netXnaMembersOnly"/>
      </label>
    </div>
  </xsl:template>
  
  <xsl:template name="membersOptionsFilterToolTip">
    <span id="membersOptionsFilterToolTip" style="cursor:default;" onmouseover="membersOptionsFilterImage.src=dropDownHoverImage.src;" onmouseout="membersOptionsFilterImage.src=dropDownImage.src;" tabindex="0">
      <img id="membersOptionsFilterImage">
        <includeAttribute name="src" item="iconPath">
          <parameter>drpdown.gif</parameter>
        </includeAttribute>
        <includeAttribute name="alt" item="dropDownImage" />
      </img>
      <label id="showAllMembersLabel" for="membersOptionsFilterImage" style="display: none;">
        <include item="membersOptionsShowAll"/>
      </label>
      <label id="filteredMembersLabel" for="membersOptionsFilterImage" style="display: none;">
        <include item="membersOptionsFiltered"/>
      </label>
    </span>
  </xsl:template>
  
  <xsl:template name="languageFilterToolTip">
    <span id="languageFilterToolTip" style="cursor:default;" onmouseover="languageFilterImage.src=dropDownHoverImage.src;" onmouseout="languageFilterImage.src=dropDownImage.src;" tabindex="0">
      <img id="languageFilterImage">
        <includeAttribute name="src" item="iconPath">
          <parameter>drpdown.gif</parameter>
        </includeAttribute>
        <includeAttribute name="alt" item="dropDownImage" />
      </img>
      <label id="showAllLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="all"/>
          </parameter>
        </include>
      </label>
      <label id="multipleLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="multiple"/>
          </parameter>
        </include>
      </label>
      <label id="vbLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="VisualBasic"/>
          </parameter>
        </include>
      </label>
      <label id="csLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="CSharp"/>
          </parameter>
        </include>
      </label>
      <label id="cLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="ManagedCPlusPlus"/>
          </parameter>
        </include>
      </label>
      <label id="jsharpLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="JSharp"/>
          </parameter>
        </include>
      </label>
      <label id="jsLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="JScript"/>
          </parameter>
        </include>
      </label>
      <label id="xamlLabel" for="languageFilterImage" style="display: none;">
        <include item="languageFilter">
          <parameter>
            <include item="XAML"/>
          </parameter>
        </include>
      </label>
    </span>
  </xsl:template>
  
  <xsl:template name="languageSpan">
    <div id="languageSpan">
      <input id="vbDeclarationCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="vbDeclarationCheckbox">
        <include item="VisualBasicDeclaration"/>
      </label>
      <br/>
      <input id="vbUsageCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="vbUsageCheckbox">
        <include item="VisualBasicUsage"/>
      </label>
      <br/>
      <input id="csCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="csCheckbox">
        <include item="CSharp"/>
      </label>
      <br/>
      <input id="cCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="cCheckbox">
        <include item="ManagedCPlusPlus"/>
      </label>
      <br/>
      <input id="jsharpCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="jsharpCheckbox">
        <include item="JSharp"/>
      </label>
      <br/>
      <input id="jsCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="jsCheckbox">
        <include item="JScript"/>
      </label>
      <br/>
      <input id="xamlCheckbox" type='checkbox' name="languageFilter" onClick="SetLanguage(this)"/>
      <label class="languageFilter" for="xamlCheckbox">
        <include item="XAML"/>
      </label>
      <br/>
    </div>
  </xsl:template>
    


  <!-- image links 
current ndppick XSLT behavior:
expandAllImage - all
dropDownImage - not namespace or derivedTypeList
dropDownHoverImage -  not namespace or derivedTypeList
collapseImage - all
expandImage - all
collapseAllImage - all
copyImage - overview (not namespace); list (only overload lists ctor, method, prop)
copyHoverImage - overview (not namespace); list (only overload lists ctor, method, prop)
  -->
  <xsl:template name="commonImages">
    <img id="collapseImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>collapse_all.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="collapseImage" />
    </img>
    <img id="expandImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>expand_all.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="expandImage" />
    </img>
    <img id="collapseAllImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>collall.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="collapseAllImage" />
    </img>
    <img id="expandAllImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>expall.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="expandAllImage" />
    </img>
    <img id="dropDownImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>drpdown.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="dropDownImage" />
    </img>
    <img id="dropDownHoverImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>drpdown_orange.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="dropDownHoverImage" />
    </img>
    <img id="copyImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>copycode.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="copyImage" />
    </img>
    <img id="copyHoverImage" style="display:none; height:0; width:0;">
      <includeAttribute name="src" item="iconPath">
        <parameter>copycodeHighlight.gif</parameter>
      </includeAttribute>
      <includeAttribute name="alt" item="copyHoverImage" />
    </img>
    
  </xsl:template>

  
</xsl:stylesheet>